﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace XNA4
{
    /// <summary>
    /// Handles all of the aspects of working with a skybox.
    /// </summary>
    public class SkyBox : DrawableGameComponent
    {
        Game game;

        /// <summary>
        /// The skybox model, which will just be a cube
        /// </summary>
        public Model skyBox;

        /// <summary>
        /// The actual skybox texture
        /// </summary>
        private TextureCube skyBoxTexture;

        /// <summary>
        /// The effect file that the skybox will use to render
        /// </summary>
        private Effect skyBoxEffect;

        /// <summary>
        /// The size of the cube, used so that we can resize the box
        /// for different sized environments.
        /// </summary>
        private float size = 50.0f;

        /// <summary>
        /// matrix
        /// </summary>
        public Matrix World;
        public Matrix View;
        public Matrix Projection;
        public Vector3 CameraPosition;

        /// <summary>
        /// Creates a new skybox
        /// </summary>
        /// <param name="skyboxTexture">the name of the skybox texture to use</param>      
        public SkyBox(Game game, string skyboxTexture)
            : base(game)
        {
            this.game = game;
            if (game.Content == null)
                return;

            skyBox = game.Content.Load<Model>("Skyboxes\\cube");
            skyBoxTexture = game.Content.Load<TextureCube>(skyboxTexture);
            skyBoxEffect = game.Content.Load<Effect>("Skyboxes\\TextureEffect");
            CameraPosition = Vector3.Zero;
        }

        /// <summary>
        /// Does the actual drawing of the skybox with our skybox effect.
        /// There is no world matrix, because we're assuming the skybox won't
        /// be moved around.  The size of the skybox can be changed with the size
        /// variable.
        /// </summary>
        /// <param name="view">The view matrix for the effect</param>
        /// <param name="projection">The projection matrix for the effect</param>
        /// <param name="cameraPosition">The position of the camera</param>
        public override void Draw(GameTime gameTime)
        {
            RasterizerState rasterizerState1 = new RasterizerState();
            rasterizerState1.CullMode = CullMode.CullClockwiseFace;
            game.GraphicsDevice.RasterizerState = rasterizerState1;

            // Go through each pass in the effect, but we know there is only one...
            foreach (EffectPass pass in skyBoxEffect.CurrentTechnique.Passes)
            {
                pass.Apply();

                // Draw all of the components of the mesh, but we know the cube really
                // only has one mesh
                foreach (ModelMesh mesh in skyBox.Meshes)
                {
                    // Assign the appropriate values to each of the parameters
                    foreach (ModelMeshPart part in mesh.MeshParts)
                    {
                        part.Effect = skyBoxEffect;
                        part.Effect.Parameters["World"].SetValue(Matrix.CreateScale(size) * Matrix.CreateTranslation(CameraPosition));
                        part.Effect.Parameters["View"].SetValue(View);
                        part.Effect.Parameters["Projection"].SetValue(Projection);
                        part.Effect.Parameters["SkyBoxTexture"].SetValue(skyBoxTexture);
                        part.Effect.Parameters["CameraPosition"].SetValue(CameraPosition);
                    }

                    // Draw the mesh with the skybox effect
                    mesh.Draw();
                }
            }
            RasterizerState rasterizerState2 = new RasterizerState();
            rasterizerState2.CullMode = CullMode.CullCounterClockwiseFace;
            game.GraphicsDevice.RasterizerState = rasterizerState2;
        }
    }
}
